@@ -14,7 +14,6 @@ from gv_proto.grpclib.interface_grpc import InterfaceStub
 from gv_utils import enums
 from gv_utils.asyncio import check_event_loop
 
-
 DATA_TYPE_EID = enums.AttId.datatypeeid
 
 METRO_PME = enums.DataTypeId.metropme
@@ -31,15 +30,18 @@ IMPUTED_PREF = enums.DataTypeId.imputedprefixe
 class Service:
     samplings = {KARRUS_RD: 1 * 60, METRO_PME: 1 * 60, TOMTOM_FCD: 1 * 60}
 
-    def __init__(self, logger, futures=None, callbacks=None):
+    def __init__(self, logger, futures=None, callbacks=None, offlineprefix=None):
         if futures is None:
             futures = []
         if callbacks is None:
             callbacks = {}
+        if offlineprefix is None:
+            offlineprefix = 'OFFLINE'
 
         self.logger = logger
         self.futures = futures
         self.callbacks = callbacks
+        self.offlineprefix = offlineprefix
         self.interface = None
         self._channel = None
         self._mainfut = None
@@ -93,7 +95,7 @@ class Service:
         success = False
         try:
             originaldatatype = datatype
-            datatype = self._is_message_from_the_past(datatype)
+            datatype = self._is_message_prefixed(datatype)
             if datatype == DATA_POINTS:
                 encode_func, args = self.__publish_data_points(data)
             elif datatype == MAPPING_ROADS_DATA_POINTS:
@@ -115,7 +117,7 @@ class Service:
                 self.logger.error(traceback.format_exc())
                 self.logger.error('An error occurred while encoding {}.'.format(originaldatatype))
             else:
-                request = PubRequest(datatype=datatype)
+                request = PubRequest(datatype=originaldatatype)
                 request.data.Pack(pbdata)
                 request.timestamp.FromSeconds(datatimestamp)
                 response = await self.interface.publish(request)
@@ -152,7 +154,7 @@ class Service:
             self.logger.info('RPC client has subscribed to {} data.'.format(datatype))
             try:
                 originaldatatype = datatype
-                datatype = self._is_message_from_the_past(datatype)
+                datatype = self._is_message_prefixed(datatype)
                 if datatype == DATA_POINTS:
                     pbdata, decode_func = self.__subscribe_data_points()
                 elif datatype == MAPPING_ROADS_DATA_POINTS:
@@ -200,11 +202,10 @@ class Service:
     def __subscribe_zones_travel_time():
         return ZonesTravelTime(), protobuf.decode_zones_travel_time
 
-    @staticmethod
-    def _is_message_from_the_past(datatype):
+    def _is_message_prefixed(self, datatype):
         sep = '-'
         splited = datatype.split(sep)
-        return datatype if splited[0] != 'PAST' else sep.join(splited[1:])
+        return datatype if splited[0] not in ('PAST', self.offlineprefix) else sep.join(splited[1:])
 
     async def _get_data_points(self, datapointeids=None, datatypeeid=None):
         lr = LocationsRequest()
@@ -231,7 +232,7 @@ class Service:
             lr.eids.eids.extend(roadeids)
         return await protobuf.decode_locations(await self.interface.get_roads(lr))
 
-    async def _get_mapping_roads_data_points(self):
+    async def _get_mapping_roads_data_points(self, datatype: str = None):
         return (await protobuf.decode_mapping(await self.interface.get_mapping_roads_data_points(MappingRequest())))[0]
 
     @staticmethod
